"use strict";

exports.__esModule = true;
exports.Consumer = exports.createConsumer = void 0;

var _react = _interopRequireDefault(require("react"));

var _differs = require("./differs");

var _raf = require("./raf");

var _context = require("./context");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function _inheritsLoose(subClass, superClass) { subClass.prototype = Object.create(superClass.prototype); subClass.prototype.constructor = subClass; subClass.__proto__ = superClass; }

var isServer = typeof navigator === 'undefined';

var ConsumerInner =
/*#__PURE__*/
function (_React$Component) {
  _inheritsLoose(ConsumerInner, _React$Component);

  function ConsumerInner(props) {
    var _this;

    _this = _React$Component.call(this) || this;
    _this.state = {};
    _this.isPure = typeof props.pure === 'undefined' ? true : props.pure;
    _this.shouldObserve = typeof props.observe === 'undefined' ? !isServer : props.observe;
    _this.scheduleUpdate = props.sync ? function () {
      return _this.update();
    } : (0, _raf.raf)(function () {
      return _this.update();
    });
    return _this;
  }

  var _proto = ConsumerInner.prototype;

  _proto.observe = function observe() {
    var _this2 = this;

    if (!this.shouldObserve) return;
    this.unobserve && this.unobserve();
    this.unobserve = this.props.atom.observe(function () {
      _this2.cancelUpdate = _this2.scheduleUpdate();
    });
    this.observedAtom = this.props.atom;
  };

  _proto.componentWillUnmount = function componentWillUnmount() {
    this.unobserve && this.unobserve();
    this.cancelUpdate && this.cancelUpdate();
    delete this.unobserve;
    delete this.cancelUpdate;
    delete this.observedAtom;
  };

  _proto.shouldComponentUpdate = function shouldComponentUpdate(nextProps, nextState) {
    if (!this.isPure) return true; // if it's <Consumer> with dynamic children, shortcut the check

    if (this.props.children !== nextProps.children) {
      return true;
    } // our state is mappedProps, this is the main optimisation


    if ((0, _differs.differs)(this.state, nextState)) {
      return true;
    } // in connect() case don't need to diff further, no extra props


    if (this.props.originalProps) {
      this.cancelUpdate && this.cancelUpdate();
      return false;
    } // in <Consumer /> case we also diff props


    if ((0, _differs.differs)(this.props, nextProps)) {
      return true;
    }

    this.cancelUpdate && this.cancelUpdate();
    return false;
  };

  _proto.componentDidUpdate = function componentDidUpdate(prevProps) {
    this.cancelUpdate && this.cancelUpdate();
  };

  _proto.update = function update() {
    this.setState({});
  };

  _proto.render = function render() {
    // do this in render, because:
    //  doing in constructor would cause memory leaks in SSR
    //  doing in componentDidMount leads to the wrong order of subscriptions
    if (!this.unobserve || this.observedAtom !== this.props.atom) this.observe();
    var _this$props = this.props,
        atom = _this$props.atom,
        originalProps = _this$props.originalProps,
        render = _this$props.render,
        children = _this$props.children;
    var actions = atom.actions;
    var mappedProps = this.state;
    return (render || children)(Object.assign({}, originalProps, {
      actions: actions
    }, mappedProps));
  };

  return ConsumerInner;
}(_react.default.Component);

ConsumerInner.getDerivedStateFromProps = function (props, state) {
  var atom = props.atom,
      originalProps = props.originalProps,
      map = props.map;
  return Object.assign({}, originalProps, map ? map(atom.get(), originalProps) : {});
};

var createConsumer = function createConsumer(AtomContext) {
  return function (props) {
    return _react.default.createElement(AtomContext.Consumer, null, function (_ref) {
      var atom = _ref.atom;
      return _react.default.createElement(ConsumerInner, _extends({}, props, {
        atom: atom
      }));
    });
  };
};

exports.createConsumer = createConsumer;
var Consumer = createConsumer(_context.AtomContext);
exports.Consumer = Consumer;